#!/bin/bash

# RHTS script for detection of avc denied services
# Different simpler approach for detection of problems with SELinux without change of runlevel
#
# 2006/10/10 Josef Kubin <jkubin@redhat.com>

OUTPUTFILE="/tmp/RHTServices4AVCDeny.log.$(date +%s)"

exec 5>&1 6>&2
exec >> "${OUTPUTFILE}" 2>&1

# source the test script helpers
. /usr/bin/rhts_environment.sh


# this is a source of order from all scripts:
# chkconfig: <runlevelsNumbers> <start priority> <shutdown priority>

# here is a list of services, which breaks testing or remote access, they won't be tested!!!
DONTEST='netplugd network single sshd auditd halt killall NetworkManager NetworkManagerDispatcher';

IFS=''

# extract lines with priority level
PRIORITY=`grep -m 1 'chkconfig:' /etc/init.d/*`

# create array of services with proper start order
STARTORDER=`echo $PRIORITY|sed 's_^\(.*\):#\s*chkconfig:\s*[^ ]\+ \+\([^ ]\+\).*$_\2 \1_'|sort -n|sed 's_^.*/__'`

# create array of services with proper stop order
STOPORDER=`echo $PRIORITY|sed 's_^\(.*\):#\s*chkconfig:\s*[^ ]\+ \+[^ ]\+ \+\([^ ]\+\).*$_\2 \1_'|sort -n|sed 's_^.*/__'`

IFS=$'\n'

# list of services, which should remain running - to avoid stop them in the end of testing
DONTSTOP=`for i in $(ls /etc/rc3.d/S*);do echo ${i#/etc/rc3.d/S??};done`

LOGFILE='/var/log/audit/audit.log'

RESULT='FAIL'

SEP='%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%'

SEP2='~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~'

# counter - how many times was problem with SELinux policy
SCORE=0

prior(){

    # save the last timestamp from the last line of the log file
    LASTIMESTAMP=`sed -n '$s/^.*audit(\([^\.]*\).*/\1/p' $LOGFILE`

    STATUS=`service $i status > /dev/null 2>&1`
    echo -n "\"service $i status\" returns $?; "

}

after(){

    # save the latest timestamp from the last line
    LATESTIMESTAMP=`sed -n '$s/^.*audit(\([^\.]*\).*/\1/p' $LOGFILE`

    # if the timestamps are different then check appended the new lines till end of log file for an "avc:  denied" substring
    # the line with $LASTIMESTAMP is excluded from testing for "avc:  denied" of course
    if [ $LASTIMESTAMP != $LATESTIMESTAMP ]; then

        # it prints only lines with 'avc:  denied' substring
	# AVCDENIED=`sed -n '/'"$LASTIMESTAMP"'/,${/'"$LASTIMESTAMP"'/b;/\<avc:  denied\>/p}' $LOGFILE`

        # It will print everything from $LASTIMESTAMP till end, on condition of occurence 'avc:  denied' substring somewhere
        # The line(s) with $LASTIMESTAMP is/are excluded from testing of course
        AVCDENIED=`sed -n '/'"$LASTIMESTAMP"'/!b;:y;n;/'"$LASTIMESTAMP"'/by;:x;$!{N;bx};/\<avc:  denied\>/p' $LOGFILE`

	if [[ $AVCDENIED ]]; then

	  (( SCORE++ ))

          IFS=''
          echo $SEP
          echo Latest appended lines from $LOGFILE
	  echo Service \"$i\" has been detected as avc denied:
          echo $SEP2
	  echo $AVCDENIED
          echo $SEP
          IFS=$'\n'
	else
	  echo `date +%s` - some lines were appended to $LOGFILE 
	fi
    fi

}

if [[ `service auditd status 2>&1|grep 'unrecognized service'` ]]; then
  echo For successful testing install firstly package \"audit\";
  exit 1;
fi

if [[ `sestatus|grep "SELinux status"` =~ "\<enabled\>" ]]; then
:
else
  echo Tested system has SELinux disabled;
  exit 2;
fi

echo $SEP2
echo '# rpm -q selinux-policy-targeted'

rpm -q selinux-policy-targeted

echo $SEP2
echo '# sestatus'
sestatus
echo $SEP2

TESTED=`for i in $STARTORDER;do [[ $DONTEST =~ "\<$i\>" ]] || echo $i;done`

echo $TESTED
echo Number of services: `echo $TESTED|wc -w`
echo $SEP2

################################################################################
###   Executive part of script #################################################
################################################################################

echo $SEP2
echo Firstly stop all unnecessary services ...
echo $SEP2

for i in $STOPORDER
do
  if [[ $DONTEST =~ "\<$i\>" ]]; then
    echo Service \"$i\" hasn\'t been stopped because it is dangerous - should be tested locally
  else

    prior

    echo -n "trying to stop service $i ..."
    service $i stop > /dev/null 2>&1
    echo "; \"service $i stop\" has returned $?"

    after

  fi
done

################################################################################

echo $SEP2
echo Now start all available services
echo $SEP2

for i in $STARTORDER
do
  if [[ $DONTEST =~ "\<$i\>" ]]; then
    echo Service \"$i\" hasn\'t been started because it is dangerous - should be tested locally
  else

  prior

  echo -n "trying to start service $i ..."
  service $i start > /dev/null 2>&1
  echo "; \"service $i start\" has returned $?"

  after

  fi
done

################################################################################

echo $SEP2
echo Stop all newly started services with exception of already running
echo $SEP2

for i in $STOPORDER
do
  if [[ $DONTSTOP =~ "\<$i\>" ]]; then
    echo Service \"$i\" must stay running
  else

    prior

    echo -n "trying to stop service $i ..."
    service $i stop > /dev/null 2>&1
    echo "; \"service $i stop\" has returned $?"

    after

  fi
done

################################################################################
### End of executive part ######################################################
################################################################################

[ $SCORE -eq 0 ] && RESULT='PASS'

exec 1>&5 2>&6

report_result $TEST $RESULT $SCORE

